/*	This file is part of the software similarity tester SIM.
	Written by Dick Grune, Vrije Universiteit, Amsterdam.
	$Id: pass2.h,v 1.3 2012-06-05 09:58:53 dick Exp $
*/

/*	Determines for each position that is part of a run, at which
	line number it starts and ends.
*/
extern void Retrieve_Runs(void);
